// This file is part of the pdr/pdx project.
// Copyright (C) 2010 Torsten Mueller, Bern, Switzerland
//
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation, either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#ifndef __DB_H
#define __DB_H

//=== Database (abstract base class) =======================================
//
// This is the base class for all database classes.
//
class Database {

	public:

	virtual	~Database ();

	virtual void Select (shared_ptr<Selection>& selection, const string& collection, const ptime& begin, const ptime& end) const throw (Xception) =0;
	virtual string GetVersion () const throw (Xception) =0;
};

//=== DBFactory ============================================================
//
// This factory creates one concrete database object according to the
// database specifications made in the configuration file. If Create()
// doesn't throw an exception the database object will immediately be
// connected and ready for use. The user should delete the object.
//
class DBFactory {

	public:

	static Database* Create (const Config& config) throw (Xception);
};

#endif
